@@ -68,7 +68,7 @@ class IndexTracker(object):
 
 def contour_worker(A):
     q, kwargs = A
-    point_maker = Point_Output_Maker_Class(**kwargs)
+    point_maker = PointOutputMakerClass(**kwargs)
     while True:
         item = q.get()
         if item is None:
@@ -89,7 +89,7 @@ def worker_def(A):
             path, iteration, out_path = item
             print(path)
             try:
-                base_class.Make_Contour_From_directory(PathDicom=path)
+                base_class.make_contour_from_directory(dicom_path=path)
                 base_class.set_iteration(iteration)
                 base_class.write_images_annotations(out_path)
                 if iteration not in final_out_dict['Iteration']:
@@ -102,7 +102,7 @@ def worker_def(A):
             q.task_done()
 
 
-class Point_Output_Maker_Class(object):
+class PointOutputMakerClass(object):
     def __init__(self, image_size_rows, image_size_cols, PixelSize, contour_dict, RS):
         self.image_size_rows, self.image_size_cols = image_size_rows, image_size_cols
         self.PixelSize = PixelSize
@@ -126,11 +126,18 @@ class Point_Output_Maker_Class(object):
                 self.contour_dict[i].append(contour)
 
 
+def poly2mask(vertex_row_coords, vertex_col_coords, shape):
+    fill_row_coords, fill_col_coords = draw.polygon(vertex_row_coords, vertex_col_coords, shape)
+    mask = np.zeros(shape, dtype=np.bool)
+    mask[fill_row_coords, fill_col_coords] = True
+    return mask
+
+
 class DicomReaderWriter:
     def __init__(self, rewrite_RT_file=False, delete_previous_rois=True, Contour_Names=None,
                  template_dir=None, get_images_mask=True, arg_max=True, create_new_RT=True, require_all_contours=True,
                  associations={},desc='',iteration=0, get_dose_output=False, flip_axes=(False, False, False), **kwargs):
-        '''
+        """
         :param rewrite_RT_file: Boolean, should we re-write the RT structure
         :param delete_previous_rois: delete the previous RTs within the structure
         :param Contour_Names: list of contour nmes
@@ -144,7 +151,7 @@ class DicomReaderWriter:
         :param get_dose_output: boolean, collect dose information
         :param flip_axes: tuple(3), axis that you want to flip, defaults to (False, False, False)
         :param kwargs:
-        '''
+        """
         self.get_dose_output = get_dose_output
         self.require_all_contours = require_all_contours
         self.flip_axes = flip_axes
@@ -209,7 +216,7 @@ class DicomReaderWriter:
             dicom_files = [i for i in files if i.endswith('.dcm')]
             if dicom_files:
                 self.all_paths.append(root)
-                self.Make_Contour_From_directory(root)
+                self.make_contour_from_directory(root)
         return None
 
     def where_are_RTs(self, ROIName):
@@ -220,10 +227,12 @@ class DicomReaderWriter:
         else:
             print('{} was not found within the set, check spelling or list all rois'.format(ROIName))
 
-    def list_rois(self):
-        print('The following ROIs were found')
-        for roi in self.all_rois:
-            print(roi)
+    def return_rois(self, print_rois=True):
+        if print_rois:
+            print('The following ROIs were found')
+            for roi in self.all_rois:
+                print(roi)
+        return self.all_rois
 
     def make_array(self, PathDicom):
         self.PathDicom = PathDicom
@@ -367,7 +376,6 @@ class DicomReaderWriter:
             [len(self.dicom_names), self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
             dtype='int8')
         for RT_key in self.RTs_in_case:
-            found_rois = {}
             ROIName_Number = self.RTs_in_case[RT_key]
             RS_struct = None
             self.structure_references = {}
@@ -380,15 +388,12 @@ class DicomReaderWriter:
                 if true_name and true_name in self.Contour_Names:
                     if RS_struct is None:
                         self.RS_struct = RS_struct = pydicom.read_file(RT_key)
-                        for contour_number in range(len(self.RS_struct.ROIContourSequence)):
-                            self.structure_references[
-                                self.RS_struct.ROIContourSequence[contour_number].ReferencedROINumber] = contour_number
-                    found_rois[true_name] = {'Hierarchy': 999, 'Name': ROI_Name, 'Roi_Number': self.RTs_in_case[RT_key][ROI_Name]}
-            for ROI_Name in found_rois.keys():
-                if found_rois[ROI_Name]['Roi_Number'] in self.structure_references:
-                    index = self.structure_references[found_rois[ROI_Name]['Roi_Number']]
-                    mask = self.Contours_to_mask(index)
-                    self.mask[..., self.Contour_Names.index(ROI_Name) + 1][mask == 1] += 1
+                    for contour_number in range(len(self.RS_struct.ROIContourSequence)):
+                        self.structure_references[
+                            self.RS_struct.ROIContourSequence[contour_number].ReferencedROINumber] = contour_number
+                    index = self.structure_references[self.RTs_in_case[RT_key][ROI_Name]]
+                    mask = self.contours_to_mask(index)
+                    self.mask[..., self.Contour_Names.index(true_name) + 1] += mask
                     self.mask[self.mask > 1] = 1
         if self.flip_axes[0]:
             self.mask = self.mask[:, :, ::-1, ...]
@@ -404,7 +409,7 @@ class DicomReaderWriter:
         self.annotation_handle.SetDirection(self.dicom_handle.GetDirection())
         return None
 
-    def Contours_to_mask(self, index):
+    def contours_to_mask(self, index):
         mask = np.zeros([len(self.dicom_names), self.image_size_rows, self.image_size_cols], dtype='int8')
         Contour_data = self.RS_struct.ROIContourSequence[index].ContourSequence
         for i in range(len(Contour_data)):
@@ -415,7 +420,7 @@ class DicomReaderWriter:
             self.col_val = matrix_points[:, 0]
             self.row_val = matrix_points[:, 1]
             z_vals = matrix_points[:, 2]
-            temp_mask = self.poly2mask(self.row_val, self.col_val, [self.image_size_rows, self.image_size_cols])
+            temp_mask = poly2mask(self.row_val, self.col_val, [self.image_size_rows, self.image_size_cols])
             temp_mask[self.row_val, self.col_val] = 0
             mask[z_vals[0], temp_mask] += 1
         mask = mask % 2
@@ -431,7 +436,7 @@ class DicomReaderWriter:
         self.key_list = self.template_dir.replace('template_RS.dcm', 'key_list.txt')
         self.RS_struct = pydicom.read_file(self.template_dir)
         print('Running off a template')
-        self.changetemplate()
+        self.change_template()
 
     def get_images(self):
         self.dicom_handle = self.reader.Execute()
@@ -445,7 +450,6 @@ class DicomReaderWriter:
         self.ArrayDicom = sitk.GetArrayFromImage(self.dicom_handle)
         self.image_size_cols, self.image_size_rows, self.image_size_z = self.dicom_handle.GetSize()
 
-
     def write_images_annotations(self, out_path):
         image_path = os.path.join(out_path, 'Overall_Data_{}_{}.nii.gz'.format(self.desciption, self.iteration))
         annotation_path = os.path.join(out_path, 'Overall_mask_{}_y{}.nii.gz'.format(self.desciption,self.iteration))
@@ -476,44 +480,48 @@ class DicomReaderWriter:
         fid = open(os.path.join(self.PathDicom, self.desciption + '_Iteration_' + self.iteration + '.txt'), 'w+')
         fid.close()
 
-    def poly2mask(self, vertex_row_coords, vertex_col_coords, shape):
-        fill_row_coords, fill_col_coords = draw.polygon(vertex_row_coords, vertex_col_coords, shape)
-        mask = np.zeros(shape, dtype=np.bool)
-        mask[fill_row_coords, fill_col_coords] = True
-        return mask
-
     def prediction_array_to_RT(self, prediction_array, output_dir, ROI_Names=None):
-        '''
+        """
         :param prediction_array: numpy array of prediction, expected shape is [#Images, Rows, Cols, #Classes + 1]
         :param output_dir: directory to pass RT structure to
         :param ROI_Names: list of ROI names equal to the number of classes
         :return:
-        '''
+        """
         if self.create_new_RT:
             self.use_template()
         assert ROI_Names is not None, 'You need to provide ROI_Names'
         assert prediction_array.shape[-1] == len(ROI_Names) + 1, 'Your last dimension of prediction array should be' \
                                                                  ' equal  to the number or ROI_names minus 1, channel' \
                                                                  ' 0 is background'
-        annotations = np.squeeze(prediction_array)
-        self.image_size_z, self.image_size_rows, self.image_size_cols = annotations.shape[:3]
+        prediction_array = np.squeeze(prediction_array)
+        contour_values = np.max(prediction_array, axis=0) # See what the maximum value is across the prediction array
+        while len(contour_values.shape) > 1:
+            contour_values = np.max(contour_values, axis=0)
+        contour_values[0] = 1  # Keep background
+        prediction_array = prediction_array[..., contour_values == 1]
+        contour_values = contour_values[1:]
+        ROI_Names = list(np.asarray(ROI_Names)[contour_values == 1])
+
+        if not ROI_Names:
+            print('RT Structure not made for {}, given prediction_array had no mask')
+        self.image_size_z, self.image_size_rows, self.image_size_cols = prediction_array.shape[:3]
         self.ROI_Names = ROI_Names
         self.output_dir = output_dir
-        if len(annotations.shape) == 3:
-            annotations = np.expand_dims(annotations, axis=-1)
+        if len(prediction_array.shape) == 3:
+            prediction_array = np.expand_dims(prediction_array, axis=-1)
         if self.flip_axes[0]:
-            annotations = annotations[:, :, ::-1, ...]
+            prediction_array = prediction_array[:, :, ::-1, ...]
         if self.flip_axes[1]:
-            annotations = annotations[:, ::-1, ...]
+            prediction_array = prediction_array[:, ::-1, ...]
         if self.flip_axes[2]:
-            annotations = annotations[::-1, ...]
-        self.annotations = annotations
-        self.Mask_to_Contours()
+            prediction_array = prediction_array[::-1, ...]
+        self.annotations = prediction_array
+        self.mask_to_contours()
 
     def with_annotations(self, annotations, output_dir, ROI_Names=None):
         self.prediction_array_to_RT(prediction_array=annotations, output_dir=output_dir, ROI_Names=ROI_Names)
 
-    def Mask_to_Contours(self):
+    def mask_to_contours(self):
         self.RefDs = self.ds
         self.PixelSize = self.dicom_handle.GetSpacing()
         current_names = []
@@ -643,7 +651,7 @@ class DicomReaderWriter:
         print('Finished!')
         return None
 
-    def changetemplate(self):
+    def change_template(self):
         keys = self.RS_struct.keys()
         for key in keys:
             # print(self.RS_struct[key].name)
@@ -708,7 +716,11 @@ class DicomReaderWriter:
             self.dose_handles.append(output)
 
     def Make_Contour_From_directory(self, PathDicom):
-        self.make_array(PathDicom)
+        print('Please move over to using make_contour_from_directory')
+        self.make_contour_from_directory(dicom_path=PathDicom)
+
+    def make_contour_from_directory(self, dicom_path):
+        self.make_array(dicom_path)
         if self.rewrite_RT_file:
             self.rewrite_RT()
         if self.get_images_mask and self.Contour_Names is not None:
@@ -730,12 +742,12 @@ class DicomReaderWriter:
         self.all_contours_exist = True
         for roi in self.Contour_Names:
             if roi not in true_rois:
-                print('Lacking {} in {}'.format(roi, PathDicom))
+                print('Lacking {} in {}'.format(roi, dicom_path))
                 print('Found {}'.format(self.rois_in_case))
                 self.all_contours_exist = False
-        if PathDicom not in self.paths_with_contours:
+        if dicom_path not in self.paths_with_contours:
             if self.all_contours_exist or not self.require_all_contours:
-                self.paths_with_contours.append(PathDicom) # Add the path that has the contours
+                self.paths_with_contours.append(dicom_path) # Add the path that has the contours
         return None
 
     def rewrite_RT(self, lstRSFile=None):
